#!/usr/bin/env python3
import os
import sys
import gradio
from dotenv import load_dotenv, find_dotenv
from langchain.chat_models import ChatOpenAI
from gpt_index import (
    SimpleDirectoryReader,
    GPTListIndex,
    GPTSimpleVectorIndex,
    LLMPredictor,
    PromptHelper
)


def get_env():
    if not os.environ.get("OPENAI_API_KEY"):
        load_dotenv(find_dotenv())
        

# parse hidden api key:
def construct_index(directory_path):
    # promt params:
    max_input_size = 4096
    num_outputs = 512
    max_chunk_overlap = 20
    chunk_size_limit = 600

    prompt_helper = PromptHelper(
        max_input_size,
        num_outputs,
        max_chunk_overlap,
        chunk_size_limit=chunk_size_limit
    )

    llm=ChatOpenAI(
        temperature=0.7,
        model_name="gpt-3.5-turbo",
        max_tokens=num_outputs
    )

    llm_predictor = LLMPredictor(llm)

    # get documents for learn:
    documents = SimpleDirectoryReader(directory_path).load_data()

    index = GPTSimpleVectorIndex(
        documents,
        llm_predictor=llm_predictor,
        prompt_helper=prompt_helper
    )

    index.save_to_disk('index.json')

    return index

def chatbot(input_text):
    index = GPTSimpleVectorIndex.load_from_disk('index.json')
    response = index.query(input_text, response_mode="compact")
    return response.response


iface = gradio.Interface(
    fn=chatbot,
    inputs=gradio.components.Textbox(lines=7, label="Enter your text"),
    outputs="text",
    title="ISPsystem custom-trained AI Chatbot"
)

index = construct_index("../docs")


def main():
    get_env()
    iface.launch(share=False)


if __name__ == "__main__":
    main()
